@@ -4,6 +4,7 @@ Field classes.
 
 import copy
 import datetime
+import enum
 import json
 import math
 import re
@@ -67,23 +68,16 @@ class Field:
         initial=None,
         error_messages=None,
         validators=(),
-        disabled=False,
     ):
         # required -- Boolean that specifies whether the field is required.
         #             True by default.
-        # widget -- A Widget class, or instance of a Widget class, that should
-        #           be used for this Field when displaying it. Each Field has a
-        #           default Widget that it'll use if you don't specify this. In
-        #           most cases, the default widget is TextInput.
         # initial -- A value to use in this Field's initial display. This value
         #            is *not* used as a fallback if data isn't given.
         # error_messages -- An optional dictionary to override the default
         #                   messages that the field will raise.
         # validators -- List of additional validators to use
-        # disabled -- Boolean that specifies whether the field is disabled, that
-        #             is its widget is shown in the form but not editable.
-        self.required, self.initial = required, initial
-        self.disabled = disabled
+        self.required = required
+        self.initial = initial
 
         messages = {}
         for c in reversed(self.__class__.__mro__):
@@ -136,16 +130,10 @@ class Field:
         For most fields, this will simply be data; FileFields need to handle it
         a bit differently.
         """
-        if self.disabled:
-            return initial
         return data
 
     def has_changed(self, initial, data):
         """Return True if data differs from initial."""
-        # Always return False if the field is disabled since self.bound_data
-        # always uses the initial value in this case.
-        if self.disabled:
-            return False
         try:
             data = self.to_python(data)
             if hasattr(self, "_coerce"):
@@ -174,12 +162,17 @@ class Field:
         return result
 
     def value_from_form_data(self, data, files, html_name):
+        # By default, all fields are expected to be present in HTML form data.
         try:
             return data[html_name]
         except KeyError as e:
-            raise FormFieldMissingError(
-                f'The "{html_name}" field is missing from the form data.'
-            ) from e
+            raise FormFieldMissingError(html_name) from e
+
+    def value_from_json_data(self, data, files, html_name):
+        if self.required and html_name not in data:
+            raise FormFieldMissingError(html_name)
+
+        return data.get(html_name, None)
 
 
 class CharField(Field):
@@ -592,11 +585,14 @@ class FileField(Field):
         return initial
 
     def has_changed(self, initial, data):
-        return not self.disabled and data is not None
+        return data is not None
 
     def value_from_form_data(self, data, files, html_name):
         return files.get(html_name)
 
+    def value_from_json_data(self, data, files, html_name):
+        return files.get(html_name)
+
 
 class ImageField(FileField):
     default_validators = [validators.validate_image_file_extension]
@@ -706,8 +702,6 @@ class BooleanField(Field):
             raise ValidationError(self.error_messages["required"], code="required")
 
     def has_changed(self, initial, data):
-        if self.disabled:
-            return False
         # Sometimes data or initial may be a string equivalent of a boolean
         # so we should run it through to_python first to get a boolean value
         return self.to_python(initial) != self.to_python(data)
@@ -729,6 +723,13 @@ class BooleanField(Field):
             "on": True,
         }.get(value)
 
+    def value_from_json_data(self, data, files, html_name):
+        # Boolean fields must be present in the JSON data
+        try:
+            return data[html_name]
+        except KeyError as e:
+            raise FormFieldMissingError(html_name) from e
+
 
 class NullBooleanField(BooleanField):
     """
@@ -773,6 +774,8 @@ class ChoiceField(Field):
         super().__init__(**kwargs)
         if hasattr(choices, "choices"):
             choices = choices.choices
+        elif isinstance(choices, enum.EnumMeta):
+            choices = [(member.value, member.name) for member in choices]
         self.choices = choices
 
     def __deepcopy__(self, memo):
@@ -883,8 +886,6 @@ class MultipleChoiceField(ChoiceField):
                 )
 
     def has_changed(self, initial, data):
-        if self.disabled:
-            return False
         if initial is None:
             initial = []
         if data is None:
@@ -944,8 +945,6 @@ class JSONField(CharField):
         super().__init__(**kwargs)
 
     def to_python(self, value):
-        if self.disabled:
-            return value
         if value in self.empty_values:
             return None
         elif isinstance(value, list | dict | int | float | JSONString):
@@ -964,8 +963,6 @@ class JSONField(CharField):
             return converted
 
     def bound_data(self, data, initial):
-        if self.disabled:
-            return initial
         if data is None:
             return None
         try:
