@@ -19,7 +19,6 @@ from plain.http.response import (
     ResponseNotAllowed,
     ResponseNotFound,
     ResponseNotModified,
-    ResponsePermanentRedirect,
     ResponseRedirect,
     ResponseServerError,
     StreamingResponse,
@@ -36,7 +35,6 @@ __all__ = [
     "ResponseBase",
     "StreamingResponse",
     "ResponseRedirect",
-    "ResponsePermanentRedirect",
     "ResponseNotModified",
     "ResponseBadRequest",
     "ResponseForbidden",
