@@ -1,17 +1,18 @@
 import codecs
 import copy
+import json
 import uuid
+from functools import cached_property
 from io import BytesIO
 from itertools import chain
 from urllib.parse import parse_qsl, quote, urlencode, urljoin, urlsplit
 
-from plain import signing
 from plain.exceptions import (
-    DisallowedHost,
     ImproperlyConfigured,
     RequestDataTooBig,
     TooManyFieldsSent,
 )
+from plain.http.cookie import unsign_cookie_value
 from plain.http.multipartparser import (
     MultiPartParser,
     MultiPartParserError,
@@ -24,15 +25,8 @@ from plain.utils.datastructures import (
     ImmutableList,
     MultiValueDict,
 )
-from plain.utils.encoding import escape_uri_path, iri_to_uri
-from plain.utils.functional import cached_property
-from plain.utils.http import is_same_domain, parse_header_parameters
-from plain.utils.regex_helper import _lazy_re_compile
-
-RAISE_ERROR = object()
-host_validation_re = _lazy_re_compile(
-    r"^([a-z0-9.-]+|\[[a-f0-9]*:[a-f0-9\.:]+\])(:[0-9]+)?$"
-)
+from plain.utils.encoding import iri_to_uri
+from plain.utils.http import parse_header_parameters
 
 
 class UnreadablePostError(OSError):
@@ -66,11 +60,11 @@ class HttpRequest:
         # A unique ID we can use to trace this request
         self.unique_id = str(uuid.uuid4())
 
-        self.GET = QueryDict(mutable=True)
-        self.POST = QueryDict(mutable=True)
-        self.COOKIES = {}
-        self.META = {}
-        self.FILES = MultiValueDict()
+        self.query_params = QueryDict(mutable=True)
+        self.data = QueryDict(mutable=True)
+        self.cookies = {}
+        self.meta = {}
+        self.files = MultiValueDict()
 
         self.path = ""
         self.path_info = ""
@@ -81,7 +75,7 @@ class HttpRequest:
 
     def __repr__(self):
         if self.method is None or not self.get_full_path():
-            return "<%s>" % self.__class__.__name__
+            return f"<{self.__class__.__name__}>"
         return f"<{self.__class__.__name__}: {self.method} {self.get_full_path()!r}>"
 
     def __getstate__(self):
@@ -101,7 +95,7 @@ class HttpRequest:
 
     @cached_property
     def headers(self):
-        return HttpHeaders(self.META)
+        return HttpHeaders(self.meta)
 
     @cached_property
     def accepted_types(self):
@@ -126,95 +120,70 @@ class HttpRequest:
             else:
                 self.encoding = self.content_params["charset"]
 
-    def _get_raw_host(self):
+    @cached_property
+    def host(self):
         """
-        Return the HTTP host using the environment or request headers. Skip
-        allowed hosts protection, so may return an insecure host.
+        Return the HTTP host using the environment or request headers.
+
+        Host validation is performed by HostValidationMiddleware, so this
+        property can safely return the host without any validation.
         """
         # We try three options, in order of decreasing preference.
-        if settings.USE_X_FORWARDED_HOST and ("HTTP_X_FORWARDED_HOST" in self.META):
-            host = self.META["HTTP_X_FORWARDED_HOST"]
-        elif "HTTP_HOST" in self.META:
-            host = self.META["HTTP_HOST"]
+        if settings.USE_X_FORWARDED_HOST and ("HTTP_X_FORWARDED_HOST" in self.meta):
+            host = self.meta["HTTP_X_FORWARDED_HOST"]
+        elif "HTTP_HOST" in self.meta:
+            host = self.meta["HTTP_HOST"]
         else:
             # Reconstruct the host using the algorithm from PEP 333.
-            host = self.META["SERVER_NAME"]
-            server_port = self.get_port()
+            host = self.meta["SERVER_NAME"]
+            server_port = self.port
             if server_port != ("443" if self.is_https() else "80"):
                 host = f"{host}:{server_port}"
         return host
 
-    def get_host(self):
-        """Return the HTTP host using the environment or request headers."""
-        host = self._get_raw_host()
-
-        # Allow variants of localhost if ALLOWED_HOSTS is empty and DEBUG=True.
-        allowed_hosts = settings.ALLOWED_HOSTS
-        if settings.DEBUG and not allowed_hosts:
-            allowed_hosts = [".localhost", "127.0.0.1", "[::1]"]
-
-        domain, port = split_domain_port(host)
-        if domain and validate_host(domain, allowed_hosts):
-            return host
-        else:
-            msg = "Invalid HTTP_HOST header: %r." % host
-            if domain:
-                msg += " You may need to add %r to ALLOWED_HOSTS." % domain
-            else:
-                msg += (
-                    " The domain name provided is not valid according to RFC 1034/1035."
-                )
-            raise DisallowedHost(msg)
-
-    def get_port(self):
+    @cached_property
+    def port(self):
         """Return the port number for the request as a string."""
-        if settings.USE_X_FORWARDED_PORT and "HTTP_X_FORWARDED_PORT" in self.META:
-            port = self.META["HTTP_X_FORWARDED_PORT"]
+        if settings.USE_X_FORWARDED_PORT and "HTTP_X_FORWARDED_PORT" in self.meta:
+            port = self.meta["HTTP_X_FORWARDED_PORT"]
         else:
-            port = self.META["SERVER_PORT"]
+            port = self.meta["SERVER_PORT"]
         return str(port)
 
     def get_full_path(self, force_append_slash=False):
-        return self._get_full_path(self.path, force_append_slash)
+        """
+        Return the full path for the request, including query string.
 
-    def get_full_path_info(self, force_append_slash=False):
-        return self._get_full_path(self.path_info, force_append_slash)
-
-    def _get_full_path(self, path, force_append_slash):
+        If force_append_slash is True, append a trailing slash if the path
+        doesn't already end with one.
+        """
         # RFC 3986 requires query string arguments to be in the ASCII range.
         # Rather than crash if this doesn't happen, we encode defensively.
+
+        def escape_uri_path(path):
+            """
+            Escape the unsafe characters from the path portion of a Uniform Resource
+            Identifier (URI).
+            """
+            # These are the "reserved" and "unreserved" characters specified in RFC
+            # 3986 Sections 2.2 and 2.3:
+            #   reserved    = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" | "$" | ","
+            #   unreserved  = alphanum | mark
+            #   mark        = "-" | "_" | "." | "!" | "~" | "*" | "'" | "(" | ")"
+            # The list of safe characters here is constructed subtracting ";", "=",
+            # and "?" according to RFC 3986 Section 3.3.
+            # The reason for not subtracting and escaping "/" is that we are escaping
+            # the entire path, not a path segment.
+            return quote(path, safe="/:@&+$,-_.!~*'()")
+
         return "{}{}{}".format(
-            escape_uri_path(path),
-            "/" if force_append_slash and not path.endswith("/") else "",
-            ("?" + iri_to_uri(self.META.get("QUERY_STRING", "")))
-            if self.META.get("QUERY_STRING", "")
+            escape_uri_path(self.path),
+            "/" if force_append_slash and not self.path.endswith("/") else "",
+            ("?" + iri_to_uri(self.meta.get("QUERY_STRING", "")))
+            if self.meta.get("QUERY_STRING", "")
             else "",
         )
 
-    def get_signed_cookie(self, key, default=RAISE_ERROR, salt="", max_age=None):
-        """
-        Attempt to return a signed cookie. If the signature fails or the
-        cookie has expired, raise an exception, unless the `default` argument
-        is provided,  in which case return that value.
-        """
-        try:
-            cookie_value = self.COOKIES[key]
-        except KeyError:
-            if default is not RAISE_ERROR:
-                return default
-            else:
-                raise
-        try:
-            value = signing.get_cookie_signer(salt=key + salt).unsign(
-                cookie_value, max_age=max_age
-            )
-        except signing.BadSignature:
-            if default is not RAISE_ERROR:
-                return default
-            else:
-                raise
-        return value
-
     def build_absolute_uri(self, location=None):
         """
         Build an absolute URI from the location and the variables available in
@@ -227,12 +196,14 @@ class HttpRequest:
         if location is None:
             # Make it an absolute url (but schemeless and domainless) for the
             # edge case that the path starts with '//'.
-            location = "//%s" % self.get_full_path()
+            location = f"//{self.get_full_path()}"
         else:
             # Coerce lazy locations.
             location = str(location)
         bits = urlsplit(location)
         if not (bits.scheme and bits.netloc):
+            current_scheme_host = f"{self.scheme}://{self.host}"
+
             # Handle the simple, most common case. If the location is absolute
             # and a scheme or host (netloc) isn't provided, skip an expensive
             # urljoin() as long as no path segments are '.' or '..'.
@@ -246,17 +217,14 @@ class HttpRequest:
                 # If location starts with '//' but has no netloc, reuse the
                 # schema and netloc from the current request. Strip the double
                 # slashes and continue as if it wasn't specified.
-                location = self._current_scheme_host + location.removeprefix("//")
+                location = current_scheme_host + location.removeprefix("//")
             else:
                 # Join the constructed URL with the provided location, which
                 # allows the provided location to apply query strings to the
                 # base path.
-                location = urljoin(self._current_scheme_host + self.path, location)
-        return iri_to_uri(location)
+                location = urljoin(current_scheme_host + self.path, location)
 
-    @cached_property
-    def _current_scheme_host(self):
-        return f"{self.scheme}://{self.get_host()}"
+        return iri_to_uri(location)
 
     def _get_scheme(self):
         """
@@ -267,15 +235,15 @@ class HttpRequest:
 
     @property
     def scheme(self):
-        if settings.SECURE_PROXY_SSL_HEADER:
+        if settings.HTTPS_PROXY_HEADER:
             try:
-                header, secure_value = settings.SECURE_PROXY_SSL_HEADER
+                header, secure_value = settings.HTTPS_PROXY_HEADER
             except ValueError:
                 raise ImproperlyConfigured(
-                    "The SECURE_PROXY_SSL_HEADER setting must be a tuple containing "
+                    "The HTTPS_PROXY_HEADER setting must be a tuple containing "
                     "two values."
                 )
-            header_value = self.META.get(header)
+            header_value = self.meta.get(header)
             if header_value is not None:
                 header_value, *_ = header_value.split(",", 1)
                 return "https" if header_value.strip() == secure_value else "http"
@@ -291,15 +259,15 @@ class HttpRequest:
     @encoding.setter
     def encoding(self, val):
         """
-        Set the encoding used for GET/POST accesses. If the GET or POST
+        Set the encoding used for query_params/data accesses. If the query_params or data
         dictionary has already been created, remove and recreate it on the
         next access (so that it is decoded correctly).
         """
         self._encoding = val
-        if hasattr(self, "GET"):
-            del self.GET
-        if hasattr(self, "_post"):
-            del self._post
+        if hasattr(self, "query_params"):
+            del self.query_params
+        if hasattr(self, "_data"):
+            del self._data
 
     def _initialize_handlers(self):
         self._upload_handlers = [
@@ -323,16 +291,15 @@ class HttpRequest:
             )
         self._upload_handlers = upload_handlers
 
-    def parse_file_upload(self, META, post_data):
-        """Return a tuple of (POST QueryDict, FILES MultiValueDict)."""
+    def parse_file_upload(self, meta, post_data):
+        """Return a tuple of (data QueryDict, files MultiValueDict)."""
         self.upload_handlers = ImmutableList(
             self.upload_handlers,
             warning=(
-                "You cannot alter upload handlers after the upload has been "
-                "processed."
+                "You cannot alter upload handlers after the upload has been processed."
             ),
         )
-        parser = MultiPartParser(META, post_data, self.upload_handlers, self.encoding)
+        parser = MultiPartParser(meta, post_data, self.upload_handlers, self.encoding)
         return parser.parse()
 
     @property
@@ -346,7 +313,7 @@ class HttpRequest:
             # Limit the maximum request data size that will be handled in-memory.
             if (
                 settings.DATA_UPLOAD_MAX_MEMORY_SIZE is not None
-                and int(self.META.get("CONTENT_LENGTH") or 0)
+                and int(self.meta.get("CONTENT_LENGTH") or 0)
                 > settings.DATA_UPLOAD_MAX_MEMORY_SIZE
             ):
                 raise RequestDataTooBig(
@@ -363,29 +330,31 @@ class HttpRequest:
         return self._body
 
     def _mark_post_parse_error(self):
-        self._post = QueryDict()
+        self._data = QueryDict()
         self._files = MultiValueDict()
 
-    def _load_post_and_files(self):
-        """Populate self._post and self._files if the content-type is a form type"""
-        if self.method != "POST":
-            self._post, self._files = (
-                QueryDict(encoding=self._encoding),
-                MultiValueDict(),
-            )
-            return
+    def _load_data_and_files(self):
+        """Populate self._data and self._files"""
+
         if self._read_started and not hasattr(self, "_body"):
             self._mark_post_parse_error()
             return
 
-        if self.content_type == "multipart/form-data":
+        if self.content_type.startswith("application/json"):
+            try:
+                self._data = json.loads(self.body)
+                self._files = MultiValueDict()
+            except json.JSONDecodeError:
+                self._mark_post_parse_error()
+                raise
+        elif self.content_type == "multipart/form-data":
             if hasattr(self, "_body"):
                 # Use already read data
                 data = BytesIO(self._body)
             else:
                 data = self
             try:
-                self._post, self._files = self.parse_file_upload(self.META, data)
+                self._data, self._files = self.parse_file_upload(self.meta, data)
             except (MultiPartParserError, TooManyFilesSent):
                 # An error occurred while parsing POST data. Since when
                 # formatting the error the request handler might access
@@ -394,12 +363,12 @@ class HttpRequest:
                 self._mark_post_parse_error()
                 raise
         elif self.content_type == "application/x-www-form-urlencoded":
-            self._post, self._files = (
+            self._data, self._files = (
                 QueryDict(self.body, encoding=self._encoding),
                 MultiValueDict(),
             )
         else:
-            self._post, self._files = (
+            self._data, self._files = (
                 QueryDict(encoding=self._encoding),
                 MultiValueDict(),
             )
@@ -413,7 +382,7 @@ class HttpRequest:
     #
     # Expects self._stream to be set to an appropriate source of bytes by
     # a corresponding request subclass (e.g. WSGIRequest).
-    # Also when request data has already been read by request.POST or
+    # Also when request data has already been read by request.data or
     # request.body, self._stream points to a BytesIO instance
     # containing that data.
 
@@ -437,6 +406,20 @@ class HttpRequest:
     def readlines(self):
         return list(self)
 
+    def get_signed_cookie(self, key, default=None, salt="", max_age=None):
+        """
+        Retrieve a cookie value signed with the SECRET_KEY.
+
+        Return default if the cookie doesn't exist or signature verification fails.
+        """
+
+        try:
+            cookie_value = self.cookies[key]
+        except KeyError:
+            return default
+
+        return unsign_cookie_value(key, cookie_value, salt, max_age, default)
+
 
 class HttpHeaders(CaseInsensitiveMapping):
     HTTP_PREFIX = "HTTP_"
@@ -470,10 +453,6 @@ class HttpHeaders(CaseInsensitiveMapping):
             return header
         return f"{cls.HTTP_PREFIX}{header}"
 
-    @classmethod
-    def to_asgi_name(cls, header):
-        return header.replace("-", "_").upper()
-
     @classmethod
     def to_wsgi_names(cls, headers):
         return {
@@ -481,13 +460,6 @@ class HttpHeaders(CaseInsensitiveMapping):
             for header_name, value in headers.items()
         }
 
-    @classmethod
-    def to_asgi_names(cls, headers):
-        return {
-            cls.to_asgi_name(header_name): value
-            for header_name, value in headers.items()
-        }
-
 
 class QueryDict(MultiValueDict):
     """
@@ -671,7 +643,7 @@ class MediaType:
         params_str = "".join(f"; {k}={v}" for k, v in self.params.items())
         return "{}{}{}".format(
             self.main_type,
-            ("/%s" % self.sub_type) if self.sub_type else "",
+            (f"/{self.sub_type}") if self.sub_type else "",
             params_str,
         )
 
@@ -708,47 +680,5 @@ def bytes_to_text(s, encoding):
         return s
 
 
-def split_domain_port(host):
-    """
-    Return a (domain, port) tuple from a given host.
-
-    Returned domain is lowercased. If the host is invalid, the domain will be
-    empty.
-    """
-    host = host.lower()
-
-    if not host_validation_re.match(host):
-        return "", ""
-
-    if host[-1] == "]":
-        # It's an IPv6 address without a port.
-        return host, ""
-    bits = host.rsplit(":", 1)
-    domain, port = bits if len(bits) == 2 else (bits[0], "")
-    # Remove a trailing dot (if present) from the domain.
-    domain = domain.removesuffix(".")
-    return domain, port
-
-
-def validate_host(host, allowed_hosts):
-    """
-    Validate the given host for this site.
-
-    Check that the host looks valid and matches a host or host pattern in the
-    given list of ``allowed_hosts``. Any pattern beginning with a period
-    matches a domain and all its subdomains (e.g. ``.example.com`` matches
-    ``example.com`` and any subdomain), ``*`` matches anything, and anything
-    else must match exactly.
-
-    Note: This function assumes that the given host is lowercased and has
-    already had the port, if any, stripped off.
-
-    Return ``True`` for a valid host, ``False`` otherwise.
-    """
-    return any(
-        pattern == "*" or is_same_domain(host, pattern) for pattern in allowed_hosts
-    )
-
-
 def parse_accept_header(header):
     return [MediaType(token) for token in header.split(",") if token.strip()]
