@@ -7,12 +7,12 @@ import re
 import sys
 import time
 from email.header import Header
+from functools import cached_property
 from http.client import responses
 from http.cookies import SimpleCookie
-from urllib.parse import urlparse
 
-from plain import signals, signing
-from plain.exceptions import DisallowedRedirect
+from plain import signals
+from plain.http.cookie import sign_cookie_value
 from plain.json import PlainJSONEncoder
 from plain.runtime import settings
 from plain.utils import timezone
@@ -72,7 +72,7 @@ class ResponseHeaders(CaseInsensitiveMapping):
             if mime_encode:
                 value = Header(value, "utf-8", maxlinelen=sys.maxsize).encode()
             else:
-                e.reason += ", HTTP response headers must be in %s format" % charset
+                e.reason += f", HTTP response headers must be in {charset} format"
                 raise
         return value
 
@@ -107,7 +107,13 @@ class ResponseBase:
     status_code = 200
 
     def __init__(
-        self, content_type=None, status=None, reason=None, charset=None, headers=None
+        self,
+        *,
+        content_type=None,
+        status_code=None,
+        reason=None,
+        charset=None,
+        headers=None,
     ):
         self.headers = ResponseHeaders(headers)
         self._charset = charset
@@ -126,9 +132,9 @@ class ResponseBase:
         self._handler_class = None
         self.cookies = SimpleCookie()
         self.closed = False
-        if status is not None:
+        if status_code is not None:
             try:
-                self.status_code = int(status)
+                self.status_code = int(status_code)
             except (ValueError, TypeError):
                 raise TypeError("HTTP status code must be an integer.")
 
@@ -181,32 +187,11 @@ class ResponseBase:
     @property
     def _content_type_for_repr(self):
         return (
-            ', "%s"' % self.headers["Content-Type"]
+            ', "{}"'.format(self.headers["Content-Type"])
             if "Content-Type" in self.headers
             else ""
         )
 
-    def __setitem__(self, header, value):
-        self.headers[header] = value
-
-    def __delitem__(self, header):
-        del self.headers[header]
-
-    def __getitem__(self, header):
-        return self.headers[header]
-
-    def has_header(self, header):
-        """Case-insensitive check for a header."""
-        return header in self.headers
-
-    __contains__ = has_header
-
-    def items(self):
-        return self.headers.items()
-
-    def get(self, header, alternate=None):
-        return self.headers.get(header, alternate)
-
     def set_cookie(
         self,
         key,
@@ -236,8 +221,8 @@ class ResponseBase:
         if expires is not None:
             if isinstance(expires, datetime.datetime):
                 if timezone.is_naive(expires):
-                    expires = timezone.make_aware(expires, datetime.timezone.utc)
-                delta = expires - datetime.datetime.now(tz=datetime.timezone.utc)
+                    expires = timezone.make_aware(expires, datetime.UTC)
+                delta = expires - datetime.datetime.now(tz=datetime.UTC)
                 # Add one second so the date matches exactly (a fraction of
                 # time gets lost between converting to a timedelta and
                 # then the date string).
@@ -271,13 +256,11 @@ class ResponseBase:
                 raise ValueError('samesite must be "lax", "none", or "strict".')
             self.cookies[key]["samesite"] = samesite
 
-    def setdefault(self, key, value):
-        """Set a header unless it has already been set."""
-        self.headers.setdefault(key, value)
-
     def set_signed_cookie(self, key, value, salt="", **kwargs):
-        value = signing.get_cookie_signer(salt=key + salt).sign(value)
-        return self.set_cookie(key, value, **kwargs)
+        """Set a cookie signed with the SECRET_KEY."""
+
+        signed_value = sign_cookie_value(key, value, salt)
+        return self.set_cookie(key, signed_value, **kwargs)
 
     def delete_cookie(self, key, path="/", domain=None, samesite=None):
         # Browsers can ignore the Set-Cookie header if the cookie doesn't use
@@ -332,14 +315,14 @@ class ResponseBase:
         signals.request_finished.send(sender=self._handler_class)
 
     def write(self, content):
-        raise OSError("This %s instance is not writable" % self.__class__.__name__)
+        raise OSError(f"This {self.__class__.__name__} instance is not writable")
 
     def flush(self):
         pass
 
     def tell(self):
         raise OSError(
-            "This %s instance cannot tell its position" % self.__class__.__name__
+            f"This {self.__class__.__name__} instance cannot tell its position"
         )
 
     # These methods partially implement a stream-like object interface.
@@ -355,7 +338,7 @@ class ResponseBase:
         return False
 
     def writelines(self, lines):
-        raise OSError("This %s instance is not writable" % self.__class__.__name__)
+        raise OSError(f"This {self.__class__.__name__} instance is not writable")
 
 
 class Response(ResponseBase):
@@ -377,8 +360,8 @@ class Response(ResponseBase):
         ]
     )
 
-    def __init__(self, content=b"", *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, content=b"", **kwargs):
+        super().__init__(**kwargs)
         # Content is a bytestring. See the `content` property methods.
         self.content = content
 
@@ -390,7 +373,7 @@ class Response(ResponseBase):
         return obj_dict
 
     def __repr__(self):
-        return "<%(cls)s status_code=%(status_code)d%(content_type)s>" % {
+        return "<%(cls)s status_code=%(status_code)d%(content_type)s>" % {  # noqa: UP031
             "cls": self.__class__.__name__,
             "status_code": self.status_code,
             "content_type": self._content_type_for_repr,
@@ -423,6 +406,10 @@ class Response(ResponseBase):
         # Create a list of properly encoded bytestrings to support write().
         self._container = [content]
 
+    @cached_property
+    def text(self):
+        return self.content.decode(self.charset or "utf-8")
+
     def __iter__(self):
         return iter(self._container)
 
@@ -454,14 +441,14 @@ class StreamingResponse(ResponseBase):
 
     streaming = True
 
-    def __init__(self, streaming_content=(), *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, streaming_content=(), **kwargs):
+        super().__init__(**kwargs)
         # `streaming_content` should be an iterable of bytestrings.
         # See the `streaming_content` property methods.
         self.streaming_content = streaming_content
 
     def __repr__(self):
-        return "<%(cls)s status_code=%(status_code)d%(content_type)s>" % {
+        return "<%(cls)s status_code=%(status_code)d%(content_type)s>" % {  # noqa: UP031
             "cls": self.__class__.__qualname__,
             "status_code": self.status_code,
             "content_type": self._content_type_for_repr,
@@ -470,8 +457,8 @@ class StreamingResponse(ResponseBase):
     @property
     def content(self):
         raise AttributeError(
-            "This %s instance has no `content` attribute. Use "
-            "`streaming_content` instead." % self.__class__.__name__
+            f"This {self.__class__.__name__} instance has no `content` attribute. Use "
+            "`streaming_content` instead."
         )
 
     @property
@@ -577,23 +564,22 @@ class FileResponse(StreamingResponse):
             self.headers["Content-Disposition"] = content_disposition
 
 
-class ResponseRedirectBase(Response):
-    allowed_schemes = ["http", "https", "ftp"]
+class ResponseRedirect(Response):
+    """HTTP redirect response"""
 
-    def __init__(self, redirect_to, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self["Location"] = iri_to_uri(redirect_to)
-        parsed = urlparse(str(redirect_to))
-        if parsed.scheme and parsed.scheme not in self.allowed_schemes:
-            raise DisallowedRedirect(
-                "Unsafe redirect to URL with protocol '%s'" % parsed.scheme
-            )
+    status_code = 302
 
-    url = property(lambda self: self["Location"])
+    def __init__(self, redirect_to, **kwargs):
+        super().__init__(**kwargs)
+        self.headers["Location"] = iri_to_uri(redirect_to)
+
+    @property
+    def url(self):
+        return self.headers["Location"]
 
     def __repr__(self):
         return (
-            '<%(cls)s status_code=%(status_code)d%(content_type)s, url="%(url)s">'
+            '<%(cls)s status_code=%(status_code)d%(content_type)s, url="%(url)s">'  # noqa: UP031
             % {
                 "cls": self.__class__.__name__,
                 "status_code": self.status_code,
@@ -603,18 +589,6 @@ class ResponseRedirectBase(Response):
         )
 
 
-class ResponseRedirect(ResponseRedirectBase):
-    """HTTP 302 response"""
-
-    status_code = 302
-
-
-class ResponsePermanentRedirect(ResponseRedirectBase):
-    """HTTP 301 response"""
-
-    status_code = 301
-
-
 class ResponseNotModified(Response):
     """HTTP 304 response"""
 
@@ -622,7 +596,7 @@ class ResponseNotModified(Response):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        del self["content-type"]
+        del self.headers["content-type"]
 
     @Response.content.setter
     def content(self, value):
@@ -658,14 +632,14 @@ class ResponseNotAllowed(Response):
 
     def __init__(self, permitted_methods, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self["Allow"] = ", ".join(permitted_methods)
+        self.headers["Allow"] = ", ".join(permitted_methods)
 
     def __repr__(self):
-        return "<%(cls)s [%(methods)s] status_code=%(status_code)d%(content_type)s>" % {
+        return "<%(cls)s [%(methods)s] status_code=%(status_code)d%(content_type)s>" % {  # noqa: UP031
             "cls": self.__class__.__name__,
             "status_code": self.status_code,
             "content_type": self._content_type_for_repr,
-            "methods": self["Allow"],
+            "methods": self.headers["Allow"],
         }
 
 
