@@ -3,37 +3,30 @@ Default Plain settings. Override these with settings in the module pointed to
 by the PLAIN_SETTINGS_MODULE environment variable.
 """
 
-from pathlib import Path
+from os import environ
 
-from plain.runtime import APP_PATH as default_app_path
+from .utils import get_app_info_from_pyproject
 
-####################
-# CORE             #
-####################
+# MARK: Core Settings
 
 DEBUG: bool = False
 
-PLAIN_TEMP_PATH: Path = default_app_path.parent / ".plain"
-
-# Hosts/domain names that are valid for this site.
-# "*" matches anything, ".example.com" matches example.com and all subdomains
-ALLOWED_HOSTS: list[str] = []
-
-# Local time zone for this installation. All choices can be found here:
-# https://en.wikipedia.org/wiki/List_of_tz_zones_by_name (although not all
-# systems may support all possibilities). This is interpreted as the default
-# user time zone.
-TIME_ZONE: str = "UTC"
-
-# Default charset to use for all Response objects, if a MIME type isn't
-# manually specified. It's used to construct the Content-Type header.
-DEFAULT_CHARSET = "utf-8"
+name, version = get_app_info_from_pyproject()
+APP_NAME: str = name
+APP_VERSION: str = version
 
 # List of strings representing installed packages.
 INSTALLED_PACKAGES: list[str] = []
 
-# Whether to append trailing slashes to URLs.
-APPEND_SLASH = True
+URLS_ROUTER: str
+
+# MARK: HTTP and Security
+
+# Hosts/domain names that are valid for this site.
+# - An empty list [] allows all hosts (useful for development).
+# - ".example.com" matches example.com and all subdomains
+# - "192.168.1.0/24" matches IP addresses in that CIDR range
+ALLOWED_HOSTS: list[str] = []
 
 # Default headers for all responses.
 DEFAULT_RESPONSE_HEADERS = {
@@ -46,10 +39,10 @@ DEFAULT_RESPONSE_HEADERS = {
     "X-Frame-Options": "DENY",
 }
 
-# Whether to redirect all non-HTTPS requests to HTTPS.
+# Whether to redirect all non-HTTPS requests to HTTPS (blanket redirect).
+# For anything more advanced (custom host, path exemptions, etc.), write
+# your own middleware.
 HTTPS_REDIRECT_ENABLED = True
-HTTPS_REDIRECT_EXEMPT = []
-HTTPS_REDIRECT_HOST = None
 
 # If your Plain app is behind a proxy that sets a header to specify secure
 # connections, AND that proxy ensures that user-submitted headers with the
@@ -74,7 +67,24 @@ SECRET_KEY: str
 # secret key rotation.
 SECRET_KEY_FALLBACKS: list[str] = []
 
-URLS_ROUTER: str
+# MARK: Internationalization
+
+# Local time zone for this installation. All choices can be found here:
+# https://en.wikipedia.org/wiki/List_of_tz_zones_by_name (although not all
+# systems may support all possibilities). This is interpreted as the default
+# user time zone.
+TIME_ZONE: str = "UTC"
+
+# Default charset to use for all Response objects, if a MIME type isn't
+# manually specified. It's used to construct the Content-Type header.
+DEFAULT_CHARSET = "utf-8"
+
+# MARK: URL Configuration
+
+# Whether to append trailing slashes to URLs.
+APPEND_SLASH = True
+
+# MARK: File Uploads
 
 # List of upload handler classes to be applied in order.
 FILE_UPLOAD_HANDLERS = [
@@ -106,46 +116,32 @@ FILE_UPLOAD_TEMP_DIR = None
 # User-defined overrides for error views by status code
 HTTP_ERROR_VIEWS: dict[int] = {}
 
-##############
-# MIDDLEWARE #
-##############
+# MARK: Middleware
 
 # List of middleware to use. Order is important; in the request phase, these
 # middleware will be applied in the order given, and in the response
 # phase the middleware will be applied in reverse order.
 MIDDLEWARE: list[str] = []
 
-###########
-# SIGNING #
-###########
+# MARK: CSRF
 
-COOKIE_SIGNING_BACKEND = "plain.signing.TimestampSigner"
-
-########
-# CSRF #
-########
-
-# Settings for CSRF cookie.
-CSRF_COOKIE_NAME = "csrftoken"
-CSRF_COOKIE_AGE = 60 * 60 * 24 * 7 * 52  # 1 year
-CSRF_COOKIE_DOMAIN = None
-CSRF_COOKIE_PATH = "/"
-CSRF_COOKIE_SECURE = True
-CSRF_COOKIE_HTTPONLY = False
-CSRF_COOKIE_SAMESITE = "Lax"
-CSRF_HEADER_NAME = "HTTP_X_CSRFTOKEN"
+# A list of trusted origins for unsafe (POST/PUT/DELETE etc.) requests.
+# These origins will be allowed regardless of the normal CSRF checks.
+# Each origin should be a full origin like "https://example.com" or "https://sub.example.com:8080"
 CSRF_TRUSTED_ORIGINS: list[str] = []
 
-###########
-# LOGGING #
-###########
+# Regex patterns for paths that should be exempt from CSRF protection
+# Examples: [r"^/api/", r"/webhooks/.*", r"/health$"]
+CSRF_EXEMPT_PATHS: list[str] = []
 
-# Custom logging configuration.
-LOGGING = {}
+# MARK: Logging
+# (Uses some custom env names in addition to PLAIN_ prefixed )
 
-###############
-# ASSETS #
-###############
+PLAIN_LOG_LEVEL: str = environ.get("PLAIN_LOG_LEVEL", "INFO")
+APP_LOG_LEVEL: str = environ.get("APP_LOG_LEVEL", "INFO")
+APP_LOG_FORMAT: str = environ.get("APP_LOG_FORMAT", "keyvalue")
+
+# MARK: Assets
 
 # Whether to redirect the original asset path to the fingerprinted path.
 ASSETS_REDIRECT_ORIGINAL = True
@@ -154,18 +150,18 @@ ASSETS_REDIRECT_ORIGINAL = True
 # Ex. "https://cdn.example.com/assets/"
 ASSETS_BASE_URL: str = ""
 
-####################
-# PREFLIGHT CHECKS #
-####################
+# MARK: Preflight Checks
 
-# List of all issues generated by system checks that should be silenced. Light
-# issues like warnings, infos or debugs will not generate a message. Silencing
-# serious issues like errors and criticals does not result in hiding the
-# message, but Plain will not stop you from e.g. running server.
-PREFLIGHT_SILENCED_CHECKS = []
+# Silence checks by name
+PREFLIGHT_SILENCED_CHECKS: list[str] = []
 
-#############
-# Templates #
-#############
+# Silence specific check results by id
+PREFLIGHT_SILENCED_RESULTS: list[str] = []
+
+# MARK: Templates
 
 TEMPLATES_JINJA_ENVIRONMENT = "plain.templates.jinja.DefaultEnvironment"
+
+# MARK: Shell
+
+SHELL_IMPORT: str = ""
